package com.weaver.seconddev.jcldoor.cmd;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.jcldoor.util.CommonUtils;
import com.weaver.seconddev.jcldoor.util.DatabaseUtils;
import com.weaver.seconddev.jcldoor.util.JucailinPortalUtils;
import com.weaver.teams.client.doc.remote.DocClientService;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.teams.security.context.UserContext;
import com.weaver.workflow.common.constant.list.listdimension.RequestListInitDimensionTabEnum;
import com.weaver.workflow.common.entity.list.api.RequestListConditionApiEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestCountInfoPAEntity;
import com.weaver.workflow.list.api.rest.publicapi.WflRequestListRest;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Period;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * 人员基本信息
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2024/3/4
 */
@Component
public class UserInfoCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(UserInfoCmd.class);

    @Autowired
    private DatabaseUtils databaseUtils;

    @RpcReference
    DocClientService docClientService;

    @RpcReference(group = "workflow")
    WflRequestListRest wflRequestListRest;

    @RpcReference
    private com.weaver.em.msg.api.rest.OaRest oaRest;

    @Autowired
    private JucailinPortalUtils jucailinPortalUtils;

    public String tenant_key = "t7akvdnf84";

    /**
     * 我在泛微信息（姓名，部门-岗位，入职天数:478天，人员交流:80人，创建文档:128个，参与群聊:128个，创建流程:239条，处理流程:439条）
     *
     * @param simpleEmployee
     * @return
     */
    public Map<String, Object> getUserInfo(SimpleEmployee simpleEmployee) {
        Map<String, Object> map = new HashMap<>(100);
        String userid = String.valueOf(simpleEmployee.getEmployeeId());
        String username = String.valueOf(simpleEmployee.getName());
        String userimageurl = "/profile/" + userid + "/personal";
        String department = "";
        String position = "";
        String days = "";
        String doccreate = "0";
        String wfcreate = "0";
        String wfdeal = "0";
        String cyqlsl = "0";
        String mtgtrs = "0";

        try {
            Map<String, Object> recordMap = new HashMap<String, Object>();
            String groupId = "weaver-ebuilder-form-service";
            String sourceType = "LOGIC";
            String tenant_key = "t7akvdnf84";

            if (StringUtils.isNotBlank(userid)) {
                String dataSql = "select   " +
                        "  a.id,   " +
                        "  a.username,   " +
                        "  a.hiredate,   " +
                        "  b.name as department,   " +
                        "  c.name as position   " +
                        "from   " +
                        "  eteams.employee a   " +
                        "  left join eteams.department b on a.department = b.id   " +
                        "  left join eteams.position c on c.id = a.position   " +
                        "where   " +
                        "  a.id = ?   " +
                        "  and a.tenant_key = ?  " +
                        "  and a.status = 'normal'   " +
                        "  and a.type = 'inside'";

                log.error("getuserIfo--dataSql:" + dataSql);

                List<String> strlist = new ArrayList<>(100);
                strlist.add(userid);
                strlist.add(tenant_key);


                List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, dataSql, sqlparam);

                List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
                if (CollectionUtils.isNotEmpty(recordList)) {
                    recordMap = recordList.get(0);
                }
                username = CommonUtils.null2String(recordMap.get("username"));
                department = CommonUtils.null2String(recordMap.get("department"));
                position = CommonUtils.null2String(recordMap.get("position"));
                String hiredate = CommonUtils.null2String(recordMap.get("hiredate"));
                if (StringUtils.isNotBlank(hiredate)) {
                    try {
                        days = calculateMinute2(hiredate);
                    } catch (Exception e) {
                        log.error("calculateMinute2-e:" + e);
                    }

                    if (StringUtils.isBlank(days)) {
                        days = String.valueOf(calculateMinute(hiredate));
                    }
                }
                //文档创建
                doccreate = getDocCreateNum(userid);
                //流程创建
                wfcreate = getWfCreateNum();
                //流程处理
                wfdeal = getWfDealNum();

                //参与群聊数量
                cyqlsl = getCyqlNum(hiredate);

                mtgtrs = getPjMtgtcs(hiredate);
            }
        } catch (Exception e) {
            log.error("getUserInfo--Exception:" + e.getMessage());
        }
        map.put("userid", userid);
        map.put("userimageurl", userimageurl);
        map.put("username", username);
        map.put("department", department);
        map.put("position", position);
        map.put("days", days);
        map.put("doccreate", doccreate);
        map.put("wfcreate", wfcreate);
        map.put("wfdeal", wfdeal);
        map.put("cyqlsl", cyqlsl);
        map.put("mtgtrs", mtgtrs);

        return map;
    }

    /**
     * 获取文档创建数量
     *
     * @param userid
     * @return
     */
    public String getDocCreateNum(String userid) {
        String num = "0";
        try {
            long employeeId = Long.parseLong(userid);
            int doccreatecount = docClientService.countByCreator(employeeId);
            num = doccreatecount + "";
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }
        return num;
    }

    /**
     * 流程创建数量
     *
     * @return
     */
    public String getWfCreateNum() {
        String num = "0";
        try {
            SimpleEmployee employee = UserContext.getCurrentUser();
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();

            WeaResult<RequestCountInfoPAEntity> RequestCountInfoPAEntity = wflRequestListRest.getRequestCountByTabId(employee, RequestListInitDimensionTabEnum.MINE_FLOWALL.getTabid(), conditionEntity);
            Long count = RequestCountInfoPAEntity.getData().getCount();
            num = count + "";
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getWfCreateNum:" + e);
        }
        return num;
    }

    /**
     * 流程处理数量
     *
     * @return
     */
    public String getWfDealNum() {
        String num = "0";
        try {
            Long count = 0L;
            SimpleEmployee employee = UserContext.getCurrentUser();
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            WeaResult<RequestCountInfoPAEntity> RequestCountInfoPAEntity = wflRequestListRest.getRequestCountByTabId(employee, RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid(), conditionEntity);
            int code = RequestCountInfoPAEntity.getCode();
            if (code == 200) {
                count = RequestCountInfoPAEntity.getData().getCount();
            }
            num = count + "";
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getWfDealNum:" + e);
        }
        return num;
    }

    /**
     * 计算日期相差天数
     *
     * @param sdate
     * @return
     * @throws ParseException
     */
    public static int calculateMinute(String sdate) throws ParseException {
        long l = 0;
        boolean flag = isTimestampFormat(sdate);
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        Date etime = new Date();
        if (flag) {
            Date stime = df.parse(sdate);
            l = etime.getTime() - stime.getTime();
        } else {
            l = etime.getTime() - new Long(sdate);
        }

        int day = (int) (l / (24 * 60 * 60 * 1000));

        return day;
    }


    public static String calculateMinute2(String sdate) throws ParseException {
        String rzsj = "";
        try {
            if (sdate.length() >= 10) {
                sdate = sdate.substring(0, 10);
            }
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String nowDate = sdf.format(new Date());
            Period period = Period.between(LocalDate.parse(sdate), LocalDate.parse(nowDate));
            int p_year = period.getYears();
            LocalDate date = LocalDate.parse(sdate);
            LocalDate p_day = date.plusYears(p_year);
            long days = ChronoUnit.DAYS.between(p_day, LocalDate.parse(nowDate));
            rzsj = p_year + "年" + days;
        } catch (Exception e) {
            log.error("calculateMinute2-e:" + e);
        }
        return rzsj;
    }

    public static boolean isTimestampFormat(String inputDate) {
        // 使用 simpleDateFormat 尝试将输入日期解析为时间戳格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            sdf.parse(inputDate);
            return true;
        } catch (Exception e) {
            return false;
        }
    }


    /**
     * 获取人员类型
     *
     * @param simpleEmployee
     * @return
     */
    public Map<String, Object> getUserType(SimpleEmployee simpleEmployee) {
        Map<String, Object> map = new HashMap<>(100);
        String type = "3";


        try {
            String field69 = "";

            SimpleEmployee employee = UserContext.getCurrentUser();
            log.error("getUserType-employee:" + employee.getEmployeeId());

            long formdata = employee.getFormdata();
            log.error("getUserType--formdata:" + formdata);

//        String groupId = "weaver-ebuilder-form-service";
            String pkey = "field69.groupId";
            //weaver-ebuilder-form-service
            String groupId = jucailinPortalUtils.getUfPropData(pkey, tenant_key);
            log.error("groupId:" + groupId);

            String sourceType = "LOGIC";

            String sql = " select field69 from eteams.Hrmemployeedefined where form_data_id='" + formdata + "'";
            log.error("getSelfServiceData--sql:" + sql);

            Map<String, Object> result = databaseUtils.execute(sourceType, groupId, sql);
            List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
            if (recordList.size() > 0) {
                // 销售 0，项目 1,技术 2，其它 3
                field69 = String.valueOf(recordList.get(0).get("field69"));
            }
            log.error("field69:" + field69);
            if (StringUtils.isNotBlank(field69)) {
                type = getfield69Type(field69);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getUserType-e:" + e);
        }

        map.put("type", type);
        return map;
    }

    private String getfield69Type(String key) {
        String type = "3";
        switch (key) {
            case "1":
                type = "0";
                break;
            case "2":
                type = "1";
                break;
            case "3":
                type = "2";
                break;
            default:
                type = "3";
        }
        return type;
    }

    private String getCyqlNum(String hiredate) {
        int count = 0;
        try {

            boolean flag = isTimestampFormat(hiredate);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd"); // 定义日期格式
            String sdate = "";

            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DAY_OF_YEAR, -1);
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String edate = dateFormat.format(calendar.getTime());

            if (flag) {
                sdate = hiredate.substring(0, 10);
            } else {
                long timestamp = new Long(hiredate); // 时间戳，单位是毫秒
                Date rzdate = new Date(timestamp); // 将时间戳转换为Date对象
                sdate = sdf.format(rzdate);
            }
            SimpleEmployee employee = UserContext.getCurrentUser();
            Long cid = employee.getCid();

            log.error("cid:" + cid);

            String pageSize = "20";
            String pageNum = "1";

            Map<String, Object> json = new HashMap<String, Object>();
            json.put("cid", cid);
            json.put("pageNum", pageNum);
            json.put("pageSize", pageSize);
            json.put("startDate", sdate);
            json.put("endDate", edate);
            log.error("getCyqlNum json:" + JSON.toJSONString(json));

            com.weaver.em.msg.api.entity.Request emrequest = new com.weaver.em.msg.api.entity.Request();
            emrequest.setUrl("/user-profile/groupSession");
            emrequest.setParamsMap(json);
            emrequest.setAsync(false);
            log.error("getCyqlNum:" + emrequest.toString());

            com.weaver.em.msg.api.entity.Response response2 = oaRest.executeIm(emrequest);
            log.error("getCyqlNum:" + response2.toString());

            String data = response2.getData();
            log.error("getCyqlNum data:" + data);
            JSONObject resObj = JSONObject.parseObject(data);
            JSONObject actionMsg = (JSONObject) resObj.get("actionMsg");
            int code = (int) actionMsg.get("code");
            if(code == 0){
                JSONObject dataObj = (JSONObject) resObj.get("data");
                count = (int) dataObj.get("count");
            }
        } catch (Exception e) {
            log.error("getCyqlNum Exception:" + e.getMessage());
            count = 0;
        }
        return count+"";

    }

    private String getPjMtgtcs(String hiredate){
        int count = 0;
        try {

            boolean flag = isTimestampFormat(hiredate);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd"); // 定义日期格式
            String sdate = "";

            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DAY_OF_YEAR, -1);
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String edate = dateFormat.format(calendar.getTime());

            if (flag) {
                sdate = hiredate.substring(0, 10);
            } else {
                long timestamp = new Long(hiredate); // 时间戳，单位是毫秒
                Date rzdate = new Date(timestamp); // 将时间戳转换为Date对象
                sdate = sdf.format(rzdate);
            }
            SimpleEmployee employee = UserContext.getCurrentUser();
            Long cid = employee.getCid();



            String pageSize = "20";
            String pageNum = "1";

            Map<String, Object> json = new HashMap<String, Object>();
            json.put("cid", cid);
            json.put("pageNum", pageNum);
            json.put("pageSize", pageSize);
            json.put("startDate", sdate);
            json.put("endDate", edate);

            log.error("getPjMtgtcs json:" + JSON.toJSONString(json));

            com.weaver.em.msg.api.entity.Request emrequest = new com.weaver.em.msg.api.entity.Request();
            emrequest.setUrl("/user-profile/dailySingleSession");
            emrequest.setParamsMap(json);
            emrequest.setAsync(false);
            log.error("getPjMtgtcs:" + emrequest.toString());

            com.weaver.em.msg.api.entity.Response response2 = oaRest.executeIm(emrequest);
            log.error("getPjMtgtcs:" + response2.toString());

            String data = response2.getData();
            log.error("getPjMtgtcs data:" + data);
            JSONObject resObj = JSONObject.parseObject(data);
            JSONObject actionMsg = (JSONObject) resObj.get("actionMsg");
            int code = (int) actionMsg.get("code");
            if(code == 0){
                JSONObject dataObj = (JSONObject) resObj.get("data");
                count = (int) dataObj.get("count");
            }
        } catch (Exception e) {
            log.error("getPjMtgtcs Exception:" + e.getMessage());
            count = 0;
        }
        return count+"";
    }


}
