package com.weaver.seconddev.jcldoor.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.weaver.common.distribution.genid.IdGenerator;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.em.msg.api.entity.Request;
import com.weaver.em.msg.api.entity.Response;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.teams.security.context.UserContext;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ChatDayUtils {

    private final Logger log = LoggerFactory.getLogger(ChatDayUtils.class);

    @RpcReference
    private com.weaver.em.msg.api.rest.OaRest oaRest;

    @Autowired
    private DatabaseUtils databaseUtils;


    public Map<String,Object> getChatDay(String tenant_key, String tablename,String pageSize,String pageNum){
        Map<String,Object> dataMap = new HashMap<String,Object>();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";

        try {
            SimpleEmployee employee = UserContext.getCurrentUser();

            String startDate = "2001-01-01";

            Calendar cal = Calendar.getInstance();
            //设置当前时间
            cal.setTime(new Date());
            //同理增加一天的方法：
            cal.add(Calendar.DATE, -1);
            String endDate = format.format(cal.getTime());

            log.error("startDate:"+startDate);
            log.error("endDate:"+endDate);
            Long cid = employee.getCid();
            log.error("ChatDayUtils--cid:"+cid);
            Long employeeId = employee.getEmployeeId();

            Map<String, Object> json = new HashMap<String, Object>();
            json.put("cid",cid);
            json.put("pageNum",pageNum);
            json.put("pageSize",pageSize);
            json.put("startDate",startDate);
            json.put("endDate",endDate);

            log.error("json:"+ JSON.toJSONString(json));

            dataMap.put("json",JSON.toJSONString(json));

            Request request = new Request();
            request.setUrl("/user-profile/dailySingleSession");
            request.setParamsMap(json);
            request.setAsync(false);

            log.error("request:"+ request.toString());
            Response response = oaRest.executeIm(request);
            log.error("getChatDay--isSuccess"+response.isSuccess());
            if (response.isSuccess()){
                dataMap.put("isSuccess","1");

                String data = response.getData();
                dataMap.put("data",data);
                if(StringUtils.isNotBlank(data)){
                    JSONObject dataJson = JSONObject.parseObject(data);
                    JSONObject actionMsg = dataJson.getJSONObject("actionMsg");
                    if(actionMsg.containsKey("code")){
                        String code = actionMsg.getString("code");
                        if("0".equals(code)){
                            JSONObject dataObject =  dataJson.getJSONObject("data");
                            if(dataObject.containsKey("data")){
                                JSONArray dataArray = dataObject.getJSONArray("data");
                                log.error("getChatDay--dataArray"+dataArray.size());


                                for(int i=0;i<dataArray.size();i++){
                                    JSONObject dataInfo = dataArray.getJSONObject(i);
                                    String userid = dataInfo.getString("userid");
                                    String amount = dataInfo.getString("amount");
                                    log.error("getChatDay--userid"+userid+"   amount:"+amount);

                                    String sql = " select id from "+tablename+" where ry=? and tenant_key=? and delete_type=0 ";
                                    List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                                    SqlParamEntity sqlParamEntity = new SqlParamEntity();
                                    sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                                    sqlParamEntity.setValue(userid+"");
                                    sqlparam.add(sqlParamEntity);

                                    sqlParamEntity = new SqlParamEntity();
                                    sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                                    sqlParamEntity.setValue(tenant_key+"");
                                    sqlparam.add(sqlParamEntity);

                                    Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                                    List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                                    if(!recordList.isEmpty()){
                                        log.error("getChatDay--!isEmpty");
                                        updateChatDayDate(userid,amount,sourceType,groupId,tenant_key,tablename);
                                    }else{
                                        log.error("getChatDay--isEmpty");
                                        insertChatDayDate(userid,amount,employeeId,sourceType,groupId,tenant_key,tablename);
                                    }
                                }
                            }
                        }
                    }
                }
            }else{
                dataMap.put("isSuccess","0");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("testSapi13:"+e);
        }
        return dataMap;
    }

    /***
     *
     * @param userid
     * @param amount
     * @param employeeId
     * @param sourceType
     * @param groupId
     * @param tenant_key
     */
    public void insertChatDayDate(String userid,String amount,long employeeId,String sourceType,String groupId,String tenant_key,String tablename){
        String code = "";
        try {
            long generate = IdGenerator.generate();
            log.error("insertChatDayDate-generate:"+generate+" userid:"+userid+" amount"+amount+" employeeId:"+employeeId);

            String sql =" insert into "+tablename+"(id,ry,amount,form_data_id,data_index,create_time,update_time,tenant_key,is_delete,creator,delete_type)  " +
                        " values (" + generate + "," + userid + ",'"+amount+"'," + generate + ",0,now(),now(),'"+tenant_key+"',0," + employeeId + ",0)";

            log.error("insertChatDayDate-sql:"+sql);
            Map<String, Object> result = databaseUtils.execute(sourceType, groupId, sql);
            if(result.containsKey("code")){
                code = String.valueOf(result.get("code"));
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("insertChatDayDate:"+e);
            code = "-100";
        }
    }

    /***
     *
     * @param userid
     * @param amount
     * @param sourceType
     * @param groupId
     * @param tenant_key
     */
    public String updateChatDayDate(String userid,String amount,String sourceType,String groupId,String tenant_key,String tablename){
        String code = "" ;
        try {
            String sql =" update "+tablename+" set amount=?,update_time=now() where ry = ? and tenant_key= ? and delete_type=0 ";
            log.error("insertChatDayDate-sql:"+sql + " amount:"+amount+" userid:"+userid);

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(amount+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(userid+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenant_key);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            if(result.containsKey("code")){
                code = String.valueOf(result.get("code"));
            }

        }catch (Exception e){
            e.printStackTrace();
            log.error("insertChatDayDate:"+e);
            code = "-100";
        }

        return code;
    }

}
